import uuid
from .api_dto import ApiDto


class Plot(ApiDto):
    """
    A plot is a definition of a Plotly figure that can be stored and shared on Wizata.
    """

    def __init__(self, plot_id=None):
        if plot_id is None:
            self.plot_id = uuid.uuid4()
        else:
            self.plot_id = plot_id
        self.name = None
        self.generatedById = None
        self.figure = None

    def api_id(self) -> str:
        return str(self.plot_id).upper()

    def endpoint(self) -> str:
        return "Plots"

    def from_json(self, obj, api=False):
        if "id" in obj.keys():
            self.plot_id = uuid.UUID(obj["id"])
        if "name" in obj.keys():
            self.name = obj["name"]
        if "figure" in obj.keys():
            self.figure = obj["figure"]
        if "generatedById" in obj.keys():
            self.generatedById = obj["generatedById"]

    def to_json(self, api=False):
        obj = {
            "id": str(self.plot_id)
        }
        if self.name is not None:
            obj["name"] = self.name
        if self.figure is not None and not api:
            obj["figure"] = self.figure
        if self.generatedById is not None:
            obj["generatedById"] = self.generatedById
        return obj



import json
import uuid
import dill
import types
from .api_dto import ApiDto


class Script(ApiDto):

    def __init__(self, script_id=None):

        # Id
        if script_id is None:
            script_id = uuid.uuid4()
        self.script_id = script_id

        # Properties
        self.name = None
        self.description = None
        self.canGeneratePlot = False
        self.canGenerateModel = False
        self.canGenerateData = False
        self.status = "draft"
        self.needExactColumnNumbers = False
        self.needExactColumnNames = False
        self.inputColumns = []
        self.outputColumns = []

        # Function properties (code)
        self.function = None

    def api_id(self) -> str:
        return str(self.script_id).upper()

    def endpoint(self) -> str:
        return "Scripts"

    def to_json(self, api=False):
        obj = {
            "id": str(self.script_id),
            "name": str(self.name),
            "description": str(self.description),
            "canGeneratePlot": str(self.canGeneratePlot),
            "canGenerateModel": str(self.canGenerateModel),
            "canGenerateData": str(self.canGenerateData),
            "status": str(self.status),
            "needExactColumnNumbers": str(self.needExactColumnNumbers),
            "needExactColumnNames": str(self.needExactColumnNames)
        }
        if api:
            obj["inputColumns"] = json.dumps(list(self.inputColumns))
            obj["outputColumns"] = json.dumps(list(self.outputColumns))
        else:
            obj["inputColumns"] = list(self.inputColumns)
            obj["outputColumns"] = list(self.outputColumns)
        return obj

    def from_json(self, obj, api=False):
        if "id" in obj.keys():
            self.script_id = uuid.UUID(obj["id"])
        if "name" in obj.keys():
            self.name = obj["name"]
        if "description" in obj.keys():
            if api:
                if obj["description"] != "None":
                    self.description = obj["description"]
            else:
                self.description = obj["description"]
        if "canGeneratePlot" in obj.keys():
            if api:
                self.canGeneratePlot = bool(obj["canGeneratePlot"])
            else:
                self.canGeneratePlot = bool(obj["canGeneratePlot"] == 'True')
        if "canGenerateModel" in obj.keys():
            if api:
                self.canGenerateModel = bool(obj["canGenerateModel"])
            else:
                self.canGenerateModel = bool(obj["canGenerateModel"] == 'True')
        if "canGenerateData" in obj.keys():
            if api:
                self.canGenerateData = bool(obj["canGenerateData"])
            else:
                self.canGenerateData = bool(obj["canGenerateData"] == 'True')
        if "status" in obj.keys():
            self.status = str(obj["status"]).lower()
        if "needExactColumnNumbers" in obj.keys():
            if api:
                self.needExactColumnNumbers = bool(obj["needExactColumnNumbers"])
            else:
                self.needExactColumnNumbers = bool(obj["needExactColumnNumbers"] == 'True')
        if "needExactColumnNames" in obj.keys():
            if api:
                self.needExactColumnNames = bool(obj["needExactColumnNames"])
            else:
                self.needExactColumnNames = bool(obj["needExactColumnNames"] == 'True')
        if "inputColumns" in obj.keys():
            self.inputColumns = obj["inputColumns"]
        if "outputColumns" in obj.keys():
            self.outputColumns = obj["outputColumns"]

    def copy(self, myfunction):
        self.function = Function()
        self.function.code = myfunction.__code__

        f_globals = myfunction.__globals__
        self.function.globals = []
        for k_global in f_globals:
            if isinstance(myfunction.__globals__[k_global], types.ModuleType):
                module = f_globals[k_global]
                self.function.globals.append({
                    "var": k_global,
                    "module": str(module.__name__)
                })


class Function:

    def __init__(self):
        self.code = None
        self.globals = None



import json
import pickle
import uuid

from .api_dto import ApiDto
from .dataframe_toolkit import DataFrameToolkit
from .mlmodel import MLModel
from .plot import Plot
from .request import Request
from .script import Script


class Execution(ApiDto):

    def __init__(self, execution_id=None):

        # Id
        if execution_id is None:
            execution_id = uuid.uuid4()
        self.execution_id = execution_id

        # Experiment information (required only for queries processed from backend)
        self.experiment_id = None
        self.experiment_type = None

        # Accept "wizard" (frontend legacy), "simulate" (model), "script" (custom function)
        self.execution_type = "wizard"

        # Inputs Properties (load)
        self.script = None
        self.request = None
        self.dataframe = None
        self.ml_model = None
        self.function = None
        self.isAnomalyDetection = False

        # Outputs Properties (save)
        self.models = []
        self.anomalies = []
        self.plots = []
        self.result_dataframe = None
        self.result_ds_dataframe = None

    def append_plot(self, figure, name="Unkwown"):
        plot = Plot()
        plot.name = name
        plot.experiment_id = self.experiment_id
        plot.figure = figure
        self.plots.append(plot)
        return plot

    def append_model(self, trained_model, input_columns, output_columns=None, has_anomalies=False, scaler=None):
        ml_model = MLModel()

        ml_model.trained_model = trained_model
        ml_model.scaler = scaler

        ml_model.input_columns = input_columns
        ml_model.output_columns = output_columns

        ml_model.has_anomalies = has_anomalies

        self.models.append(ml_model)
        return ml_model

    def api_id(self) -> str:
        return str(self.execution_id).upper()

    def endpoint(self) -> str:
        return "Executions"

    def to_json(self, result=False, api=False):
        """
        Convert to a json version of Execution definition.
        By default, use DS API format.
        :param result: if set as True, use format for python client.
        :param api: if set as True, use format for backend API.
        """
        obj = {
            "id": str(self.execution_id)
        }
        if self.experiment_id is not None and api:
            obj["experiment_id"] = str(self.experiment_id)
        if self.request is not None and not result:
            if api:
                obj["request"] = json.dumps(self.request.to_json())
            else:
                obj["request"] = self.request.to_json()
        if self.dataframe is not None and not result and not api:
            obj["dataframe"] = DataFrameToolkit.convert_to_json(self.dataframe)
        if self.script is not None and not result:
            if api:
                obj["script_id"] = str(self.script.script_id)
            else:
                obj["script"] = self.script.to_json()
        if self.ml_model is not None and not result:
            if api:
                obj["model_id"] = str(self.ml_model.model_id)
            else:
                obj["ml_model"] = self.ml_model.to_json()
        if self.function is not None and not result:
            obj["function"] = self.function
        if self.isAnomalyDetection and not result:
            obj["isAnomalyDetection"] = str(True)
        if self.plots is not None and not api:
            plots_ids = []
            for plot in self.plots:
                plots_ids.append({"id": str(plot.plot_id), "name": plot.name})
            obj["plots"] = plots_ids
        if self.models is not None and not api:
            models_json = []
            for ml_model in self.models:
                models_json.append(ml_model.get_sample_payload())
            obj["models"] = models_json
        if self.anomalies is not None and not api:
            obj["anomaliesList"] = self.anomalies
        if self.result_dataframe is not None and not api:
            obj["result"] = DataFrameToolkit.convert_to_json(self.result_dataframe)
        if self.result_ds_dataframe is not None and not api:
            obj["result_ds_dataframe_id"] = str(self.result_ds_dataframe.df_id)
        return obj

    def from_json(self, obj, api=False):
        if "id" in obj.keys():
            self.execution_id = uuid.UUID(obj["id"])
        if "request" in obj.keys():
            self.request = Request()
            self.request.from_json(obj["request"])
        if "dataframe" in obj.keys():
            self.dataframe = DataFrameToolkit.convert_from_json(obj["dataframe"])
        if "script" in obj.keys():
            self.script = Script()
            self.script.from_json(obj["script"], api=api)
        if "ml_model" in obj.keys():
            self.ml_model = MLModel()
            self.ml_model.from_json(obj["ml_model"], api=api)
        if "function" in obj.keys() and obj:
            self.function = obj["function"]
        if "isAnomalyDetection" in obj.keys() and obj["isAnomalyDetection"] == 'True':
            self.isAnomalyDetection = True

    def to_pickle(self):
        return pickle.dumps(self)
